diff -Naur channels/chan_iax2.c nchannels/chan_iax2.c
--- channels/chan_iax2.c	2007-03-19 16:02:35.000000000 +0900
+++ nchannels/chan_iax2.c	2007-03-19 16:03:17.000000000 +0900
@@ -5336,8 +5336,22 @@
 	}
 	if (ies->encmethods)
 		ast_set_flag(p, IAX_ENCRYPTED | IAX_KEYPOPULATED);
-	if (!res)
+	if (!res) {
+		struct ast_var_t *var;
+		/* Add remote vars */
+		AST_LIST_TRAVERSE(&p->owner->varshead, var, entries) {
+			if (!strncmp(ast_var_name(var), "~IAX2~", strlen("~IAX2~"))) {
+				char tmp[256];
+				int i;
+				/* Automatically divide the value up into sized chunks */
+				for (i = 0; i < strlen(ast_var_value(var)); i += 255 - (strlen(ast_var_name(var)) - strlen("~IAX2~") + 1)) {
+					snprintf(tmp, sizeof(tmp), "%s=%s", ast_var_name(var) + strlen("~IAX2~"), ast_var_value(var) + i);
+					iax_ie_append_str(&ied, IAX_IE_VARIABLE, tmp);
+				}
+			}
+		}
 		res = send_command(p, AST_FRAME_IAX, IAX_COMMAND_AUTHREP, 0, ied.buf, ied.pos, -1);
+	}
 	return res;
 }
 
@@ -6515,23 +6529,22 @@
 	return 1;
 }
 
-static int acf_iaxvar_read(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len)
+static char * acf_iaxvar_read(struct ast_channel *chan, char *cmd, char *data, char *buf, size_t len)
 {
 	const char *value;
 	char tmp[256];
 	snprintf(tmp, sizeof(tmp), "~IAX2~%s", data);
 	value = pbx_builtin_getvar_helper(chan, tmp);
 	ast_copy_string(buf, value ? value : "", len);
-	return 0;
+	return buf;
 }
 
-static int acf_iaxvar_write(struct ast_channel *chan, const char *cmd, char *varname, const char *value)
+void acf_iaxvar_write(struct ast_channel *chan, char *cmd, char *varname, const char *value)
 {
 	char tmp[256];
 	/* Inherit forever */
 	snprintf(tmp, sizeof(tmp), "__~IAX2~%s", varname);
 	pbx_builtin_setvar_helper(chan, tmp, value);
-	return 0;
 }
 
 static struct ast_custom_function iaxvar_function = {
@@ -7497,6 +7510,18 @@
 							ast_set_flag(&iaxs[fr->callno]->state, IAX_STATE_STARTED);
 							if(!(c = ast_iax2_new(fr->callno, AST_STATE_RING, format)))
 								iax2_destroy(fr->callno);
+							else if (ies.vars) {
+								struct ast_variable *var, *prev = NULL;
+								char tmp[256];
+								for (var = ies.vars; var; var = var->next) {
+									if (prev)
+										free(prev);
+									prev = var;
+									snprintf(tmp, sizeof(tmp), "__~IAX2~%s", var->name);
+									pbx_builtin_setvar_helper(c, tmp, var->value);
+								}
+								ies.vars = NULL;
+							}
 						} else {
 							ast_set_flag(&iaxs[fr->callno]->state, IAX_STATE_TBD);
 							/* If this is a TBD call, we're ready but now what...  */
